@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -13,9 +13,9 @@ __docformat__ = 'restructuredtext'
 
 import numpy as N
 from mvpa.base import externals
-externals.exists("scipy", raiseException=True)
 
-from scipy.linalg import lstsq
+if externals.exists("scipy", raiseException=True):
+    from scipy.linalg import lstsq
 
 from mvpa.clfs.base import Classifier
 
