@@ -126,7 +126,7 @@ class SVM(_SVM):
 
     _clf_internals = _SVM._clf_internals + [ 'sg', 'retrainable' ]
 
-    if externals.exists('sg >= 0.6.4'):
+    if externals.exists('sg ge 0.6.4'):
         _KERNELS['linear'] = (shogun.Kernel.LinearKernel, (), LinearSVMWeights)
 
     # Some words of wisdom from shogun author:
@@ -318,7 +318,7 @@ class SVM(_SVM):
                             self._kernel_type(self.__traindata, self.__traindata,
                                               *kargs)
 
-            if externals.exists('sg >= 0.6.4'):
+            if externals.exists('sg ge 0.6.4'):
                  kernel.set_normalizer(shogun.Kernel.IdentityKernelNormalizer())
 
             newkernel = True
@@ -484,7 +484,7 @@ class SVM(_SVM):
                 _setdebug(kernel_test, 'Kernels')
 
                 custk_args = ([self.__traindata, testdata], [])[
-                    int(externals.exists('sg >= 0.6.4'))]
+                    int(externals.exists('sg ge 0.6.4'))]
                 if __debug__:
                     debug("SG__",
                           "Re-creating custom testing kernel giving "
