@@ -1,12 +1,12 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
 #   copyright and license terms.
 #
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
-"""PyMVPA datasets and helper classes such as mappers, splitters
+"""PyMVPA datasets and helper classes such as splitters
 
 Module Description
 ==================
@@ -22,12 +22,10 @@ The mvpa.datasets module contains the following modules:
 .. packagetree::
    :style: UML
 
-:group Generic Datasets: base, mapped, masked, meta
-:group Specialized Datasets: nifti, eep
-:group Mappers: mapper, maskmapper
-:group Metrics: metric
-:group Splitters: splitter, nfoldsplitter
-:group Miscellaneous: miscfx
+:group Generic Datasets: base mapped masked meta
+:group Specialized Datasets: nifti channel eep event
+:group Splitters: splitter
+:group Miscellaneous: miscfx miscfx_sp
 
 
 """
