@@ -26,8 +26,10 @@ from mvpa.misc.exceptions import DatasetError
 from mvpa.misc.support import idhash as idhash_
 from mvpa.base.dochelpers import enhancedDocString, table2string
 
+from mvpa.base import warning
+
 if __debug__:
-    from mvpa.base import debug, warning
+    from mvpa.base import debug
 
     def _validate_indexes_uniq_sorted(seq, fname, item):
         """Helper function to validate that seq contains unique sorted values
@@ -58,9 +60,9 @@ class Dataset(object):
         `applyMapper`
       - `Mutators`: `permuteLabels`
 
-    Important: labels assumed to be immutable, i.e. noone should modify
+    Important: labels assumed to be immutable, i.e. no one should modify
     them externally by accessing indexed items, ie something like
-    ``dataset.labels[1] += "_bad"`` should not be used. If a label has
+    ``dataset.labels[1] += 100`` should not be used. If a label has
     to be modified, full copy of labels should be obtained, operated on,
     and assigned back to the dataset, otherwise dataset.uniquelabels
     would not work.  The same applies to any other attribute which has
@@ -156,13 +158,17 @@ class Dataset(object):
           samples : ndarray
             2d array (samples x features)
           labels
-            An array or scalar value defining labels for each samples
+            An array or scalar value defining labels for each samples.
+            Generally `labels` should be numeric, unless `labels_map`
+            is used
           labels_map : None or bool or dict
-            Map from labels into literal names. If is None or True,
-            the mapping is computed, from labels which must be literal.
-            If is False, no mapping is computed. If dict -- mapping is
-            verified and taken, labels get remapped. Dict must map
-            literal -> number
+            Map original labels into numeric labels.  If True, the
+            mapping is computed if labels are literal.  If is False,
+            no mapping is computed. If dict instance -- provided
+            mapping is verified and applied.  If you want to have
+            labels_map just be present given already numeric labels,
+            just assign labels_map dictionary to existing dataset
+            instance
           chunks
             An array or scalar value defining chunks for each sample
 
@@ -219,7 +225,7 @@ class Dataset(object):
         """Dataset attriibutes."""
 
         # store samples (and possibly transform/reshape/retype them)
-        if not samples == None:
+        if not samples is None:
             if __debug__:
                 if lcl_data.has_key('samples'):
                     debug('DS',
@@ -233,7 +239,7 @@ class Dataset(object):
         #       ie if no labels present -- assign arange
         #   MH: don't think this is necessary -- or is there a use case?
         # labels
-        if not labels == None:
+        if not labels is None:
             if __debug__:
                 if lcl_data.has_key('labels'):
                     debug('DS',
@@ -286,8 +292,8 @@ class Dataset(object):
         labels_ = N.asarray(lcl_data['labels'])
         labels_map_known = lcl_dsattr.has_key('labels_map')
         if labels_map is True:
-            # need to composte labels_map
-            if labels_.dtype.char == 'S' or not labels_map_known:
+            # need to compose labels_map
+            if labels_.dtype.char == 'S': # or not labels_map_known:
                 # Create mapping
                 ulabels = list(Set(labels_))
                 ulabels.sort()
@@ -504,18 +510,21 @@ class Dataset(object):
         labels = self.labels
         nsamples = self.nsamples
 
-        lastseen = [None for attr in attributes_to_track]
+        lastseen = none = [None for attr in attributes_to_track]
         transitions = []
 
-        for i in xrange(nsamples):
-            current = [_data[attr][i] for attr in attributes_to_track]
+        for i in xrange(nsamples+1):
+            if i < nsamples:
+                current = [_data[attr][i] for attr in attributes_to_track]
+            else:
+                current = none
             if lastseen != current:
                 # transition point
                 new_transitions = range(max(0, i-prior),
                                         min(nsamples-1, i+post)+1)
                 if affected_labels is not None:
-                    new_transitions = filter(lambda i: labels[i] in affected_labels,
-                                             new_transitions)
+                    new_transitions = [labels[i] for i in new_transitions
+                                       if i in affected_labels]
                 transitions += new_transitions
                 lastseen = current
 
@@ -589,6 +598,10 @@ class Dataset(object):
         if not (uniques == sorted_ids).all():
             raise DatasetError, "Samples IDs are not unique."
 
+        # Check if labels as not literal
+        if N.asanyarray(_data['labels'].dtype.char == 'S'):
+            warning('Labels for dataset %s are literal, should be numeric. '
+                    'You might like to use labels_map argument.' % self)
 
     def _expandSampleAttribute(self, attr, attr_name):
         """If a sample attribute is given as a scalar expand/repeat it to a
@@ -946,9 +959,13 @@ class Dataset(object):
         return out
 
 
-    def copy(self):
+    def copy(self, deep=True):
         """Create a copy (clone) of the dataset, by fully copying current one
 
+        :Keywords:
+          deep : bool
+            deep flag is provided to __init__ for
+            copy_{samples,data,dsattr}. By default full copy is done.
         """
         # create a new object of the same type it is now and NOT only Dataset
         out = super(Dataset, self).__new__(self.__class__)
@@ -975,16 +992,14 @@ class Dataset(object):
             incremental order. If not such, in non-optimized code
             selectFeatures would verify the order and sort
 
-        Returns a new Dataset object with a view of the original
-        samples array (no copying is performed).
+        Returns a new Dataset object with a copy of corresponding features
+		from the original samples array.
 
         WARNING: The order of ids determines the order of features in
         the returned dataset. This might be useful sometimes, but can
         also cause major headaches! Order would is verified when
         running in non-optimized code (if __debug__)
         """
-        ids = copy.deepcopy(ids)
-        
         if ids is None and groups is None:
             raise ValueError, "No feature selection specified."
 
@@ -1003,6 +1018,7 @@ class Dataset(object):
         # XXX set sort default to True, now sorting has to be explicitely
         # disabled and warning is not necessary anymore
         if sort:
+            ids = copy.deepcopy(ids)
             ids.sort()
         elif __debug__ and 'CHECK_DS_SORTED' in debug.active:
             from mvpa.misc.support import isSorted
@@ -1024,7 +1040,7 @@ class Dataset(object):
         else:
             new_dsattr = self._dsattr
 
-        # create a new object of the same type it is now and NOT onyl Dataset
+        # create a new object of the same type it is now and NOT only Dataset
         dataset = super(Dataset, self).__new__(self.__class__)
 
         # now init it: to make it work all Dataset contructors have to accept
@@ -1656,3 +1672,256 @@ Dataset._registerAttribute("labels",  "_data", abbr='L', hasunique=True)
 Dataset._registerAttribute("chunks",  "_data", abbr='C', hasunique=True)
 # samples ids (already unique by definition)
 Dataset._registerAttribute("origids",  "_data", abbr='I', hasunique=False)
+
+
+
+# XXX This is the place to redo the Dataset base class in a more powerful, yet
+# simpler way. The basic goal is to allow for all kinds of attributes:
+#
+# 1) Samples attributes (per-sample full)
+# 2) Features attributes (per-feature stuff)
+#
+# 3) Dataset attributes (per-dataset stuff)
+#
+# Use cases:
+#
+#     1) labels and chunks -- goal: it should be possible to have multivariate
+#     labels, e.g. to specify targets for a neural network output layer
+#
+#     2) feature binding/grouping -- goal: easily define ROIs in datasets, or
+#     group/mark various types of feature so they could be selected or
+#     discarded all together
+#
+#     3) Mappers, or chains of them (this should be possible already, but could
+#     be better integrated to make applyMapper() obsolete).
+#
+#
+# Perform distortion correction on __init__(). The copy contructor
+# implementation should move into a separate classmethod.
+#
+# Think about implementing the current 'clever' attributes in terms of one-time
+# properties as suggested by Fernando on nipy-devel.
+
+# ...
+
+from mvpa.misc.state import ClassWithCollections, Collection
+from mvpa.misc.attributes import SampleAttribute, FeatureAttribute, \
+        DatasetAttribute
+
+# Remaining public interface of Dataset
+class _Dataset(ClassWithCollections):
+    """The successor of Dataset.
+    """
+    # placeholder for all three basic collections of a Dataset
+    # put here to be able to check whether the AttributesCollector already
+    # instanciated a particular collection
+    # XXX maybe it should not do this at all for Dataset
+    sa = None
+    fa = None
+    dsa = None
+
+    # storage of samples in a plain NumPy array for fast access
+    samples = None
+
+    def __init__(self, samples, sa=None, fa=None, dsa=None):
+        """
+        This is the generic internal constructor. Its main task is to allow
+        for a maximum level of customization during dataset construction,
+        including fast copy construction.
+
+        Parameters
+        ----------
+        samples : ndarray
+          Data samples.
+        sa : Collection
+          Samples attributes collection.
+        fa : Collection
+          Features attributes collection.
+        dsa : Collection
+          Dataset attributes collection.
+        """
+        # init base class
+        ClassWithCollections.__init__(self)
+
+        # Internal constructor -- users focus on init* Methods
+
+        # Every dataset needs data (aka samples), completely data-driven
+        # analyses might not even need labels, so this is the only mandatory
+        # argument
+        # XXX add checks
+        self.samples = samples
+
+        # Everything else in a dataset (except for samples) is organized in
+        # collections
+        # copy attributes from source collections (scol) into target
+        # collections (tcol)
+        for scol, tcol in ((sa, self.sa),
+                           (fa, self.fa),
+                           (dsa, self.dsa)):
+            # make sure we have the target collection
+            if tcol is None:
+                # XXX maybe use different classes for the collections
+                # but currently no reason to do so
+                tcol = Collection(owner=self)
+
+            # transfer the attributes
+            if not scol is None:
+                for name, attr in scol.items.iteritems():
+                    # this will also update the owner of the attribute
+                    # XXX discuss the implications of always copying
+                    tcol.add(copy.copy(attr))
+
+
+    @classmethod
+    def initSimple(klass, samples, labels, chunks):
+        # use Numpy convention
+        """
+        One line summary.
+
+        Long description.
+
+        Parameters
+        ----------
+        samples : ndarray
+          The two-dimensional samples matrix.
+        labels : ndarray
+        chunks : ndarray
+
+        Returns
+        -------
+        blah blah
+
+        Notes
+        -----
+        blah blah
+
+        See Also
+        --------
+        blah blah
+
+        Examples
+        --------
+        blah blah
+        """
+        # Demo user contructor
+
+        # compile the necessary samples attributes collection
+        labels_ = SampleAttribute(name='labels')
+        labels_.value = labels
+        chunks_ = SampleAttribute(name='chunks')
+        chunks_.value = chunks
+
+        # feels strange that one has to give the name again
+        # XXX why does items have to be a dict when each samples
+        # attr already knows its name
+        sa = Collection(items={'labels': labels_, 'chunks': chunks_})
+
+        # common checks should go into __init__
+        return klass(samples, sa=sa)
+
+
+    def getNSamples( self ):
+        """Currently available number of patterns.
+        """
+        return self.samples.shape[0]
+
+
+    def getNFeatures( self ):
+        """Number of features per pattern.
+        """
+        return self.samples.shape[1]
+
+
+#
+#    @property
+#    def idhash(self):
+#        pass
+#
+#
+#    def idsonboundaries(self, prior=0, post=0,
+#                        attributes_to_track=['labels', 'chunks'],
+#                        affected_labels=None,
+#                        revert=False):
+#        pass
+#
+#
+#    def summary(self, uniq=True, stats=True, idhash=False, lstats=True,
+#                maxc=30, maxl=20):
+#        pass
+#
+#
+#    def summary_labels(self, maxc=30, maxl=20):
+#        pass
+#
+#
+#    def __iadd__(self, other):
+#        pass
+#
+#
+#    def __add__( self, other ):
+#        pass
+#
+#
+#    def copy(self):
+#        pass
+#
+#
+#    def selectFeatures(self, ids=None, sort=True, groups=None):
+#        pass
+#
+#
+#    def applyMapper(self, featuresmapper=None, samplesmapper=None,
+#                    train=True):
+#        pass
+#
+#
+#    def selectSamples(self, ids):
+#        pass
+#
+#
+#    def index(self, *args, **kwargs):
+#        pass
+#
+#
+#    def select(self, *args, **kwargs):
+#        pass
+#
+#
+#    def where(self, *args, **kwargs):
+#        pass
+#
+#
+#    def __getitem__(self, *args):
+#        pass
+#
+#
+#    def permuteLabels(self, status, perchunk=True, assure_permute=False):
+#        pass
+#
+#
+#    def getRandomSamples(self, nperlabel):
+#        pass
+#
+#
+#    def getLabelsMap(self):
+#        pass
+#
+#
+#    def setLabelsMap(self, lm):
+#        pass
+#
+#
+#    def setSamplesDType(self, dtype):
+#        pass
+#
+#
+#    def defineFeatureGroups(self, definition):
+#        pass
+#
+#
+#    def convertFeatureIds2FeatureMask(self, ids):
+#        pass
+#
+#
+#    def convertFeatureMask2FeatureIds(self, mask):
+#        pass
