@@ -69,7 +69,7 @@ class DatasetMeasure(ClassWithCollections):
     def __init__(self, transformer=None, null_dist=None, **kwargs):
         """Does nothing special.
 
-        :Parameter:
+        :Parameters:
           transformer: Functor
             This functor is called in `__call__()` to perform a final
             processing step on the to be returned dataset measure. If None,
@@ -192,6 +192,13 @@ class DatasetMeasure(ClassWithCollections):
             prefixes.append("null_dist=%s" % self.__null_dist)
         return super(DatasetMeasure, self).__repr__(prefixes=prefixes)
 
+    def untrain(self):
+        """'Untraining' Measure
+
+        Some derived classes might used classifiers, so we need to
+        untrain those
+        """
+        pass
 
     @property
     def null_dist(self):
@@ -434,6 +441,12 @@ class Sensitivity(FeaturewiseDatasetMeasure):
         self.__clf = clf
 
 
+    def untrain(self):
+        """Untrain corresponding classifier for Sensitivity
+        """
+        if self.__clf is not None:
+            self.__clf.untrain()
+
     @property
     def feature_ids(self):
         """Return feature_ids used by the underlying classifier
@@ -500,6 +513,13 @@ class CombinedFeaturewiseDatasetMeasure(FeaturewiseDatasetMeasure):
         return sensitivities
 
 
+    def untrain(self):
+        """Untrain CombinedFDM
+        """
+        if self.__analyzers is not None:
+            for anal in self.__analyzers:
+                anal.untrain()
+
     def _setAnalyzers(self, analyzers):
         """Set the analyzers
         """
@@ -562,6 +582,14 @@ class SplitFeaturewiseDatasetMeasure(FeaturewiseDatasetMeasure):
 
         self.__insplit_index = insplit_index
 
+
+    def untrain(self):
+        """Untrain SplitFeaturewiseDatasetMeasure
+        """
+        if self.__analyzer is not None:
+            self.__analyzer.untrain()
+
+
     def _call(self, dataset):
         # local bindings
         analyzer = self.__analyzer
@@ -632,6 +660,14 @@ class BoostedClassifierSensitivityAnalyzer(Sensitivity):
         """Analyzer to use for basic classifiers within boosted classifier"""
 
 
+    def untrain(self):
+        """Untrain BoostedClassifierSensitivityAnalyzer
+        """
+        if self.__analyzer is not None:
+            self.__analyzer.untrain()
+        self.__combined_analyzer.untrain()
+
+
     def _call(self, dataset):
         analyzers = []
         # create analyzers
@@ -691,6 +727,12 @@ class ProxyClassifierSensitivityAnalyzer(Sensitivity):
         """Analyzer to use for basic classifiers within boosted classifier"""
 
 
+    def untrain(self):
+        super(ProxyClassifierSensitivityAnalyzer, self).untrain()
+        if self.__analyzer is not None:
+            self.__analyzer.untrain()
+
+
     def _call(self, dataset):
         # OPT: local bindings
         clfclf = self.clf.clf
